{% load i18n %}
{% load navactive %}

{% if perms.taxsystem.create_access %}
    <li class="nav-item">
        <a class="nav-link py-0" href="{% url 'taxsystem:add_corp' %}">
            <span class="btn btn-success">{% translate 'Add Corporation' %}</span>
        </a>
    </li>
    <li class="nav-item">
        <a class="nav-link py-0" href="{% url 'taxsystem:add_alliance' %}">
            <span class="btn btn-success">{% translate 'Add Alliance' %}</span>
        </a>
    </li>
{% endif %}

{% if perms.taxsystem.manage_own_corp or perms.taxsystem.manage_corps or perms.taxsystem.manage_own_alliance or perms.taxsystem.manage_alliances %}
    {% if corporation_id %}
        <li class="nav-item">
            <a class="nav-link py-0" href="{% url 'taxsystem:manage_owner' owner_id=corporation_id %}">
                <span class="btn btn-success">{% translate 'Manage Corporation' %}</span>
            </a>
        </li>
    {% endif %}

    {% if alliance_id %}
        <li class="nav-item">
            <a class="nav-link py-0" href="{% url 'taxsystem:manage_owner' owner_id=alliance_id %}">
                <span class="btn btn-success">{% translate 'Manage Alliance' %}</span>
            </a>
        </li>
    {% endif %}
{% endif %}
