{% load i18n %}

<button type="button" class="btn btn-danger btn-sm btn-square me-2"
    data-bs-toggle="modal"
    data-tooltip-toggle="taxsystem-tooltip"
    data-character-id="{{ member.character_id }}"
    data-title="{% translate 'Delete Member' %} <b>{{ member.character_name }}</b>"
    data-text="{% translate 'Are you sure to Confirm' %}?"
    data-bs-target="#members-delete-member"
    data-action="{% url 'taxsystem:delete_member' member_pk=member.pk corporation_id=member.owner.eve_corporation.corporation_id %}"

    title="{% translate 'Delete Member' %}"
    >
    <span class="fas fa-trash"></span>
</button>
