{% load i18n %}

<button type="button" class="btn btn-warning btn-sm btn-square me-2"
    data-bs-toggle="modal"
    data-tooltip-toggle="taxsystem-tooltip"
    data-character-id="{{ character.character.character_id }}"
    data-title="{% translate 'Undo Payment' %}"
    data-text="{% translate 'Undo Payment' %} {{ payment.amount }} ISK {% translate 'from' %} {{ payment.account.user.username }}"
    data-bs-target="#payments-undo"
    data-action="{% url 'taxsystem:undo_payment' payment_pk=payment.pk owner_id=payment.owner_id %}"

    title="{% translate 'Undo Payment' %}"
    >
    <span class="fas fa-undo"></span>
</button>
