{% load i18n %}
{% load humanize %}

<button type="button" class="btn btn-danger btn-sm btn-square me-2"
    data-bs-toggle="modal"
    data-tooltip-toggle="taxsystem-tooltip"
    data-title="{% translate 'Delete Filter' %} <b>{{ filter.filter_type }} - {{ filter.value|intcomma }}</b>"
    data-text="{% translate 'Are you sure to Confirm' %}?"
    data-bs-target="#filter-set-delete-filter"
    data-action="{% url 'taxsystem:delete_filter' filter_pk=filter.pk owner_id=filter.filter_set.owner.eve_id %}"

    title="{% translate 'Delete Filter' %}"
    >
    <span class="fas fa-trash"></span>
</button>
